# Copyright (c) 2021 RiceXL

from RiceXL.RiceExcel import RiceExcel
from RiceXL.RiceFont import RiceFont
from RiceXL.RiceBorder import RiceBorder
from RiceXL.RiceAlignment import RiceAlignment
from RiceXL.RicePattern import RicePattern
from RiceXL.RiceUtils import get_color_rgb, get_color_rgb_by_index, get_color_index_by_rgb

__AUTHOR__ = 'SNAKE'
__AUTHOR_EMAIL__ = 'admin@wangbaishi.com'
__LICENSE__ = 'MIT'
__VERSION__ = '0.1.0'
__PYTHON__ = '3.6'